@@ -6,7 +6,7 @@
  *
  * u_response.c: response related functions defintions
  *
- * Copyright 2015-2020 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2015-2022 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -23,9 +23,8 @@
  *
  */
 #include <string.h>
-
-#include "u_private.h"
-#include "ulfius.h"
+#include <u_private.h>
+#include <ulfius.h>
 
 static char * ulfius_generate_cookie_header(const struct _u_cookie * cookie) {
   char * attr_expires = NULL, * attr_max_age = NULL, * attr_domain = NULL, * attr_path = NULL;
@@ -597,7 +596,7 @@ int ulfius_set_response_properties(struct _u_response * response, ...) {
 
   if (response != NULL) {
     va_start(vl, response);
-    for (option = va_arg(vl, uint); option != U_OPT_NONE && ret == U_OK; option = va_arg(vl, uint)) {
+    for (option = va_arg(vl, unsigned int); option != U_OPT_NONE && ret == U_OK; option = va_arg(vl, unsigned int)) {
       switch (option) {
         case U_OPT_STATUS:
           response->status = va_arg(vl, long);
@@ -614,7 +613,7 @@ int ulfius_set_response_properties(struct _u_response * response, ...) {
         case U_OPT_AUTH_REALM:
           str_value = va_arg(vl, const char *);
           o_free(response->auth_realm);
-          if (o_strlen(str_value)) {
+          if (!o_strnullempty(str_value)) {
             response->auth_realm = o_strdup(str_value);
           } else {
             response->auth_realm = NULL;
@@ -715,3 +714,40 @@ int ulfius_add_header_to_response(struct _u_response * response, const char * ke
     return U_ERROR_PARAMS;
   }
 }
+
+char * ulfius_export_response_http(const struct _u_response * response) {
+  char * out = NULL, * header;
+  const char * value = NULL, ** keys = NULL;
+  unsigned int i;
+
+  if (response != NULL) {
+    out = msprintf("HTTP/1.1 %ld\r\n", response->status);
+      
+    keys = u_map_enum_keys(response->map_header);
+    for (i=0; keys != NULL && keys[i] != NULL; i++) {
+      value = u_map_get(response->map_header, keys[i]);
+      if (value != NULL) {
+        out = mstrcatf(out, "%s: %s\r\n", keys[i], value);
+      } else {
+        out = mstrcatf(out, "%s:\r\n", keys[i]);
+      }
+    }
+
+    if (response->nb_cookies && !u_map_has_key_case(response->map_header, "Set-Cookie")) {
+      for (i=0; i<response->nb_cookies; i++) {
+        header = ulfius_generate_cookie_header(&response->map_cookie[i]);
+        out = mstrcatf(out, "Set-Cookie: %s\r\n", header);
+        o_free(header);
+      }
+    }
+    if (response->binary_body_length) {
+      out = mstrcatf(out, "Content-Length: %zu\r\n", response->binary_body_length);
+    }
+    out = mstrcatf(out, "\r\n");
+      
+    if (response->binary_body_length) {
+      out = mstrcatf(out, "%.*s\r\n", response->binary_body_length, response->binary_body);
+    }
+  }
+  return out;
+}
