@@ -6,7 +6,7 @@
  *
  * u_send_request.c: send request related functions defintions
  *
- * Copyright 2015-2020 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2015-2022 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -31,14 +31,15 @@
   #define _POSIX_C_SOURCE 200112L
 #endif
 
-#include "u_private.h"
-#include "ulfius.h"
+#include <u_private.h>
+#include <ulfius.h>
 
 #ifndef U_DISABLE_CURL
 
-#include <stdlib.h>
-#include <ctype.h>
 #include <curl/curl.h>
+
+#include <ctype.h>
+#include <stdlib.h>
 #include <string.h>
 
 #define U_ACCEPT_HEADER  "Accept-Encoding"
@@ -58,6 +59,7 @@ struct tm * gmtime_r(const time_t* t, struct tm* r) {
 }
 #endif
 
+
 /**
  * Internal structure used to store temporarily the response body
  */
@@ -104,7 +106,8 @@ static size_t ulfius_write_body(void * contents, size_t size, size_t nmemb, void
 static size_t write_header(void * buffer, size_t size, size_t nitems, void * user_data) {
 
   struct _u_response * response = (struct _u_response *) user_data;
-  char * header = (char *)buffer, * key, * value, * saveptr, * tmp;
+  char * header = (char *)buffer, * saveptr, * tmp;
+  const char * key, * value;
 
   if (o_strchr(header, ':') != NULL) {
     if (response->map_header != NULL) {
@@ -122,7 +125,7 @@ static size_t write_header(void * buffer, size_t size, size_t nitems, void * use
         o_free(tmp);
       }
     }
-  } else if (o_strlen(trimwhitespace(header)) > 0) {
+  } else if (!o_strnullempty(trimwhitespace(header))) {
     // Expecting the HTTP/x.x header
     if (response->protocol != NULL) {
       o_free(response->protocol);
@@ -195,7 +198,7 @@ int ulfius_send_http_streaming_request(const struct _u_request * request,
   CURLcode res;
   CURL * curl_handle = NULL;
   struct curl_slist * header_list = NULL, * cookies_list = NULL;
-  char * key_esc = NULL, * value_esc = NULL, * cookie = NULL, * cookies = NULL, * header = NULL, * fp = "?", * np = "&", * url = NULL;
+  char * key_esc = NULL, * value_esc = NULL, * cookie = NULL, * cookies = NULL, * header = NULL, fp = '?', np = '&', * url = NULL;
   const char * value = NULL, ** keys = NULL;
   int i, has_params = 0, ret, exit_loop;
   struct _u_request * copy_request = NULL;
@@ -316,10 +319,10 @@ int ulfius_send_http_streaming_request(const struct _u_request * request,
                   value_esc = curl_easy_escape(curl_handle, value, 0);
                   if (value_esc != NULL) {
                     if (!has_params) {
-                      url = mstrcatf(url, "%s%s=%s", fp, key_esc, value_esc);
+                      url = mstrcatf(url, "%c%s=%s", fp, key_esc, value_esc);
                       has_params = 1;
                     } else {
-                      url = mstrcatf(url, "%s%s=%s", np, key_esc, value_esc);
+                      url = mstrcatf(url, "%c%s=%s", np, key_esc, value_esc);
                     }
                     curl_free(value_esc);
                   } else {
@@ -328,10 +331,10 @@ int ulfius_send_http_streaming_request(const struct _u_request * request,
                   }
                 } else {
                   if (!has_params) {
-                    url = mstrcatf(url, "%s%s", fp, key_esc);
+                    url = mstrcatf(url, "%c%s", fp, key_esc);
                     has_params = 1;
                   } else {
-                    url = mstrcatf(url, "%s%s", np, key_esc);
+                    url = mstrcatf(url, "%c%s", np, key_esc);
                   }
                 }
                 curl_free(key_esc);
@@ -364,7 +367,7 @@ int ulfius_send_http_streaming_request(const struct _u_request * request,
                     if (!i) {
                       copy_request->binary_body = mstrcatf(copy_request->binary_body, "%s=%s", key_esc, value_esc);
                     } else {
-                      copy_request->binary_body = mstrcatf(copy_request->binary_body, "%s%s=%s", np, key_esc, value_esc);
+                      copy_request->binary_body = mstrcatf(copy_request->binary_body, "%c%s=%s", np, key_esc, value_esc);
                     }
                     copy_request->binary_body_length = o_strlen(copy_request->binary_body);
                   } else {
@@ -376,7 +379,7 @@ int ulfius_send_http_streaming_request(const struct _u_request * request,
                   if (!i) {
                     copy_request->binary_body = mstrcatf(copy_request->binary_body, "%s", key_esc);
                   } else {
-                    copy_request->binary_body = mstrcatf(copy_request->binary_body, "%s%s", np, key_esc);
+                    copy_request->binary_body = mstrcatf(copy_request->binary_body, "%c%s", np, key_esc);
                   }
                   copy_request->binary_body_length = o_strlen(copy_request->binary_body);
                 }
