@@ -6,7 +6,7 @@
  *
  * ulfius.c: framework functions definitions
  *
- * Copyright 2015-2020 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2015-2022 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -22,18 +22,18 @@
  * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
-
-#include <string.h>
-#include <ctype.h>
-#include <stdlib.h>
-#include <pthread.h>
-#include "u_private.h"
-#include "ulfius.h"
+#include <u_private.h>
+#include <ulfius.h>
 
 #ifndef U_DISABLE_CURL
 #include <curl/curl.h>
 #endif
 
+#include <ctype.h>
+#include <pthread.h>
+#include <stdlib.h>
+#include <string.h>
+
 /** Define mock yder functions when yder is disabled **/
 #ifdef U_DISABLE_YDER
 int y_init_logs(const char * app, const unsigned long init_mode, const unsigned long init_level, const char * init_log_file, const char * message) {
@@ -66,10 +66,11 @@ int y_close_logs(void) {
  * Fill a map with the key/values specified
  */
 #if MHD_VERSION >= 0x00097002
-static enum MHD_Result ulfius_fill_map_check_utf8(void * cls, enum MHD_ValueKind kind, const char * key, const char * value) {
+static enum MHD_Result ulfius_fill_map_check_utf8(void * cls, enum MHD_ValueKind kind, const char * key, const char * value)
 #else
-static int ulfius_fill_map_check_utf8(void * cls, enum MHD_ValueKind kind, const char * key, const char * value) {
+static int ulfius_fill_map_check_utf8(void * cls, enum MHD_ValueKind kind, const char * key, const char * value)
 #endif
+  {
   char * tmp;
   int res;
   UNUSED(kind);
@@ -103,10 +104,11 @@ static int ulfius_fill_map_check_utf8(void * cls, enum MHD_ValueKind kind, const
  * Fill a map with the key/values specified
  */
 #if MHD_VERSION >= 0x00097002
-static enum MHD_Result ulfius_fill_map(void * cls, enum MHD_ValueKind kind, const char * key, const char * value) {
+static enum MHD_Result ulfius_fill_map(void * cls, enum MHD_ValueKind kind, const char * key, const char * value)
 #else
-static int ulfius_fill_map(void * cls, enum MHD_ValueKind kind, const char * key, const char * value) {
+static int ulfius_fill_map(void * cls, enum MHD_ValueKind kind, const char * key, const char * value)
 #endif
+  {
   char * tmp;
   int res;
   UNUSED(kind);
@@ -137,10 +139,11 @@ static int ulfius_fill_map(void * cls, enum MHD_ValueKind kind, const char * key
  * return true if the endpoind has valid parameters
  */
 #if MHD_VERSION >= 0x00097002
-static enum MHD_Result ulfius_is_valid_endpoint(const struct _u_endpoint * endpoint, int to_delete) {
+static enum MHD_Result ulfius_is_valid_endpoint(const struct _u_endpoint * endpoint, int to_delete)
 #else
-static int ulfius_is_valid_endpoint(const struct _u_endpoint * endpoint, int to_delete) {
+static int ulfius_is_valid_endpoint(const struct _u_endpoint * endpoint, int to_delete)
 #endif
+  {
   if (endpoint != NULL) {
     if (ulfius_equals_endpoints(endpoint, ulfius_empty_endpoint())) {
       // Should be the last endpoint of the list to close it
@@ -277,6 +280,27 @@ static int ulfius_get_body_from_response(struct _u_response * response, void **
   }
 }
 
+void mhd_redirect_log(void * arg, const char * fmt, va_list ap) {
+  UNUSED(arg);
+  va_list args_cpy, args_cpy2;
+  size_t out_len = 0;
+  char * out = NULL, * new_fmt;
+
+  va_copy(args_cpy, ap);
+  va_copy(args_cpy2, ap);
+  new_fmt = msprintf("MHD - %s", fmt);
+  out_len = vsnprintf(NULL, 0, new_fmt, args_cpy);
+  out = o_malloc((out_len)*sizeof(char));
+  if (out != NULL) {
+    vsnprintf(out, (out_len), new_fmt, args_cpy2);
+    y_log_message(Y_LOG_LEVEL_ERROR, out);
+    o_free(out);
+  }
+  o_free(new_fmt);
+  va_end(args_cpy);
+  va_end(args_cpy2);
+}
+
 /**
  * mhd_request_completed
  * function used to clean data allocated after a web call is complete
@@ -311,12 +335,13 @@ void mhd_request_completed (void *cls, struct MHD_Connection *connection,
 #if MHD_VERSION >= 0x00097002
 static enum MHD_Result mhd_iterate_post_data (void * coninfo_cls, enum MHD_ValueKind kind, const char * key,
                                               const char * filename, const char * content_type,
-                                              const char * transfer_encoding, const char * data, uint64_t off, size_t size) {
+                                              const char * transfer_encoding, const char * data, uint64_t off, size_t size)
 #else
 static int mhd_iterate_post_data (void * coninfo_cls, enum MHD_ValueKind kind, const char * key,
                                   const char * filename, const char * content_type,
-                                  const char * transfer_encoding, const char * data, uint64_t off, size_t size) {
+                                  const char * transfer_encoding, const char * data, uint64_t off, size_t size)
 #endif
+{
   struct connection_info_struct * con_info = coninfo_cls;
   size_t data_size = size, cur_size;
   char * filename_param = NULL, * data_concat = NULL;
@@ -338,7 +363,7 @@ static int mhd_iterate_post_data (void * coninfo_cls, enum MHD_ValueKind kind, c
     
     do {
       if (con_info->u_instance->check_utf8) {
-        if (utf8_check(key, o_strlen(key)) != NULL || data == NULL || utf8_check(data, o_strlen(data)) != NULL || (filename != NULL && utf8_check(filename, o_strlen(filename)) != NULL)) {
+        if (utf8_check(key, o_strlen(key)) != NULL || data == NULL || utf8_check(data, size) != NULL || (filename != NULL && utf8_check(filename, o_strlen(filename)) != NULL)) {
           break;
         }
       }
@@ -347,7 +372,7 @@ static int mhd_iterate_post_data (void * coninfo_cls, enum MHD_ValueKind kind, c
         break;
       }
       
-      if (off + size > con_info->max_post_param_size) {
+      if (con_info->max_post_param_size && off + size > con_info->max_post_param_size) {
         data_size = con_info->max_post_param_size - off;
       }
       
@@ -356,45 +381,26 @@ static int mhd_iterate_post_data (void * coninfo_cls, enum MHD_ValueKind kind, c
         if (!u_map_has_key((struct _u_map *)con_info->request->map_post_body, filename_param) && u_map_put((struct _u_map *)con_info->request->map_post_body, filename_param, filename) != U_OK) {
           y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error u_map_put filename value");
         }
-        cur_data = u_map_get((struct _u_map *)con_info->request->map_post_body, key);
-        cur_size = u_map_get_length((struct _u_map *)con_info->request->map_post_body, key);
-        if (cur_data != NULL) {
-          if (off) {
-            if (u_map_put_binary((struct _u_map *)con_info->request->map_post_body, key, data, cur_size, data_size) != U_OK) {
-              ret = MHD_NO;
-              break;
-            }
-          } else {
-            if (u_map_put_binary((struct _u_map *)con_info->request->map_post_body, key, ",", cur_size, 1) != U_OK ||
-                u_map_put_binary((struct _u_map *)con_info->request->map_post_body, key, data, cur_size+1, 1) != U_OK) {
-              ret = MHD_NO;
-              break;
-            }
+      }
+      cur_data = u_map_get((struct _u_map *)con_info->request->map_post_body, key);
+      cur_size = u_map_get_length((struct _u_map *)con_info->request->map_post_body, key);
+      if (cur_data != NULL) {
+        if (off) {
+          if (u_map_put_binary((struct _u_map *)con_info->request->map_post_body, key, data, cur_size, data_size) != U_OK) {
+            ret = MHD_NO;
+            break;
           }
         } else {
-          if (u_map_put_binary((struct _u_map *)con_info->request->map_post_body, key, data, 0, data_size) != U_OK) {
+          if (u_map_put_binary((struct _u_map *)con_info->request->map_post_body, key, ",", cur_size, 1) != U_OK ||
+              u_map_put_binary((struct _u_map *)con_info->request->map_post_body, key, data, cur_size+1, data_size) != U_OK) {
             ret = MHD_NO;
             break;
           }
         }
       } else {
-        cur_data = u_map_get((struct _u_map *)con_info->request->map_post_body, key);
-        cur_size = u_map_get_length((struct _u_map *)con_info->request->map_post_body, key);
-        if (cur_data != NULL) {
-          if (off) {
-            data_concat = msprintf("%s%s", cur_data, data);
-          } else {
-            data_concat = msprintf("%s,%s", cur_data, data);
-          }
-          if (u_map_put((struct _u_map *)con_info->request->map_post_body, key, data_concat) != U_OK) {
-            ret = MHD_NO;
-            break;
-          }
-        } else {
-          if (u_map_put((struct _u_map *)con_info->request->map_post_body, key, data) != U_OK) {
-            ret = MHD_NO;
-            break;
-          }
+        if (u_map_put_binary((struct _u_map *)con_info->request->map_post_body, key, data, 0, data_size) != U_OK) {
+          ret = MHD_NO;
+          break;
         }
       }
       
@@ -424,7 +430,7 @@ static enum MHD_Result ulfius_webservice_dispatcher (void * cls,
                                                      const char * version,
                                                      const char * upload_data,
                                                      size_t * upload_data_size,
-                                                     void ** con_cls) {
+                                                     void ** con_cls)
 #else
 static int ulfius_webservice_dispatcher (void * cls,
                                          struct MHD_Connection * connection,
@@ -433,8 +439,9 @@ static int ulfius_webservice_dispatcher (void * cls,
                                          const char * version,
                                          const char * upload_data,
                                          size_t * upload_data_size,
-                                         void ** con_cls) {
+                                         void ** con_cls)
 #endif
+{
   struct _u_endpoint * endpoint_list = ((struct _u_instance *)cls)->endpoint_list, ** current_endpoint_list = NULL, * current_endpoint = NULL;
   struct connection_info_struct * con_info = * con_cls;
   int mhd_ret = MHD_NO, callback_ret = U_OK, i, close_loop = 0, inner_error = U_OK, mhd_response_flag;
@@ -522,8 +529,8 @@ static int ulfius_webservice_dispatcher (void * cls,
 
     // Set POST Processor if content-type is properly set
     if (content_type != NULL && 
-       (0 == o_strncmp(MHD_HTTP_POST_ENCODING_FORM_URLENCODED, content_type, o_strlen(MHD_HTTP_POST_ENCODING_FORM_URLENCODED)) ||
-        0 == o_strncmp(MHD_HTTP_POST_ENCODING_MULTIPART_FORMDATA, content_type, o_strlen(MHD_HTTP_POST_ENCODING_MULTIPART_FORMDATA)))) {
+       ((con_info->u_instance->allowed_post_processor&U_POST_PROCESS_URL_ENCODED && 0 == o_strncmp(MHD_HTTP_POST_ENCODING_FORM_URLENCODED, content_type, o_strlen(MHD_HTTP_POST_ENCODING_FORM_URLENCODED))) ||
+        (con_info->u_instance->allowed_post_processor&U_POST_PROCESS_MULTIPART_FORMDATA && 0 == o_strncmp(MHD_HTTP_POST_ENCODING_MULTIPART_FORMDATA, content_type, o_strlen(MHD_HTTP_POST_ENCODING_MULTIPART_FORMDATA))))) {
       con_info->has_post_processor = 1;
       con_info->post_processor = MHD_create_post_processor (connection, ULFIUS_POSTBUFFERSIZE, mhd_iterate_post_data, (void *) con_info);
       if (NULL == con_info->post_processor) {
@@ -727,7 +734,7 @@ static int ulfius_webservice_dispatcher (void * cls,
                         MHD_add_response_header (mhd_response,
                                                  "Sec-WebSocket-Protocol",
                                                  protocol);
-                        if (o_strlen(extension)) {
+                        if (!o_strnullempty(extension)) {
                           MHD_add_response_header (mhd_response,
                                                    "Sec-WebSocket-Extensions",
                                                    extension);
@@ -1042,7 +1049,7 @@ static int ulfius_webservice_dispatcher (void * cls,
  *
  */
 static struct MHD_Daemon * ulfius_run_mhd_daemon(struct _u_instance * u_instance, const char * key_pem, const char * cert_pem, const char * root_ca_perm) {
-  unsigned int mhd_flags = MHD_USE_THREAD_PER_CONNECTION;
+  unsigned int mhd_flags = MHD_USE_THREAD_PER_CONNECTION | MHD_USE_ERROR_LOG;
   int index;
 
 #ifdef DEBUG
@@ -1056,24 +1063,28 @@ static struct MHD_Daemon * ulfius_run_mhd_daemon(struct _u_instance * u_instance
 #endif
 
   if (u_instance->mhd_daemon == NULL) {
-    struct MHD_OptionItem mhd_ops[8];
+    struct MHD_OptionItem mhd_ops[9];
 
     // Default options
-    mhd_ops[0].option = MHD_OPTION_NOTIFY_COMPLETED;
-    mhd_ops[0].value = (intptr_t)mhd_request_completed;
+    mhd_ops[0].option = MHD_OPTION_EXTERNAL_LOGGER;
+    mhd_ops[0].value = (intptr_t)mhd_redirect_log;
     mhd_ops[0].ptr_value = NULL;
 
+    mhd_ops[1].option = MHD_OPTION_NOTIFY_COMPLETED;
+    mhd_ops[1].value = (intptr_t)mhd_request_completed;
+    mhd_ops[1].ptr_value = NULL;
+
 #if MHD_VERSION >= 0x00095208
     // If bind_address6 is specified, listen only to IPV6 addresses
     if (u_instance->bind_address6 != NULL) {
-      mhd_ops[1].option = MHD_OPTION_SOCK_ADDR;
-      mhd_ops[1].value = 0;
-      mhd_ops[1].ptr_value = (void *)u_instance->bind_address6;
+      mhd_ops[2].option = MHD_OPTION_SOCK_ADDR;
+      mhd_ops[2].value = 0;
+      mhd_ops[2].ptr_value = (void *)u_instance->bind_address6;
       mhd_flags |= MHD_USE_IPv6;
     } else {
-      mhd_ops[1].option = MHD_OPTION_SOCK_ADDR;
-      mhd_ops[1].value = 0;
-      mhd_ops[1].ptr_value = (void *)u_instance->bind_address;
+      mhd_ops[2].option = MHD_OPTION_SOCK_ADDR;
+      mhd_ops[2].value = 0;
+      mhd_ops[2].ptr_value = (void *)u_instance->bind_address;
       // Default network stack is listening to IPV4 only
       if ((u_instance->network_type & U_USE_IPV4) && (u_instance->network_type & U_USE_IPV6)) {
         // If u_instance->network_type & U_USE_ALL, listen to IPV4 and IPV6 addresses
@@ -1084,16 +1095,16 @@ static struct MHD_Daemon * ulfius_run_mhd_daemon(struct _u_instance * u_instance
       }
     }
 #else
-    mhd_ops[1].option = MHD_OPTION_SOCK_ADDR;
-    mhd_ops[1].value = 0;
-    mhd_ops[1].ptr_value = (void *)u_instance->bind_address;
+    mhd_ops[2].option = MHD_OPTION_SOCK_ADDR;
+    mhd_ops[2].value = 0;
+    mhd_ops[2].ptr_value = (void *)u_instance->bind_address;
 #endif
 
-    mhd_ops[2].option = MHD_OPTION_URI_LOG_CALLBACK;
-    mhd_ops[2].value = (intptr_t)ulfius_uri_logger;
-    mhd_ops[2].ptr_value = NULL;
+    mhd_ops[3].option = MHD_OPTION_URI_LOG_CALLBACK;
+    mhd_ops[3].value = (intptr_t)ulfius_uri_logger;
+    mhd_ops[3].ptr_value = NULL;
 
-    index = 3;
+    index = 4;
 
     if (key_pem != NULL && cert_pem != NULL) {
       // HTTPS parameters
@@ -1660,11 +1671,12 @@ static int internal_ulfius_init_instance(struct _u_instance * u_instance,
   pthread_mutexattr_t mutexattr;
 #endif
 #if MHD_VERSION >= 0x00095208
-  if (u_instance != NULL && port > 0 && port < 65536 && (bind_address4 == NULL || bind_address6 == NULL) && (network_type & U_USE_ALL)) {
+  if (u_instance != NULL && port > 0 && port < 65536 && (bind_address4 == NULL || bind_address6 == NULL) && (network_type & U_USE_ALL))
 #else
 UNUSED(network_type);
-  if (u_instance != NULL && port > 0 && port < 65536) {
+  if (u_instance != NULL && port > 0 && port < 65536)
 #endif
+  {
     u_instance->mhd_daemon = NULL;
     u_instance->status = U_STATUS_STOP;
     u_instance->port = port;
@@ -1722,6 +1734,7 @@ UNUSED(network_type);
 #else
     u_instance->websocket_handler = NULL;
 #endif
+    u_instance->allowed_post_processor = U_POST_PROCESS_URL_ENCODED|U_POST_PROCESS_MULTIPART_FORMDATA;
     return U_OK;
   } else {
     return U_ERROR_PARAMS;
@@ -1842,7 +1855,7 @@ static char to_hex(char code) {
 char * ulfius_url_encode(const char * str) {
   char * pstr = (char*)str, * buf = NULL, * pbuf = NULL;
   if (str != NULL) {
-    buf = o_malloc(strlen(str) * 3 + 1);
+    buf = o_malloc(o_strlen(str) * 3 + 1);
     if (buf != NULL) {
       pbuf = buf;
       while (* pstr) {
@@ -1876,7 +1889,7 @@ char * ulfius_url_encode(const char * str) {
 char * ulfius_url_decode(const char * str) {
   char * pstr = (char*)str, * buf = NULL, * pbuf = NULL;
   if (str != NULL) {
-    buf = o_malloc(strlen(str) + 1);
+    buf = o_malloc(o_strlen(str) + 1);
     if (buf != NULL) {
       pbuf = buf;
       while (* pstr) {
